import QtQuick 1.1
import qb.components 1.0
import BasicUIControls 1.0;

StyledRectangle {
	id: root
	width: 114
	height: 65

	property string kpiPostfix: subLabelText
	property alias subLabelText: subLabel.text
	property int stateId: -1
	property real temperature: 0
	property bool selected: false
	radius: 6
	topLeftRadiusRatio: 0
	topRightRadiusRatio: 0
	bottomLeftRadiusRatio: 0
	bottomRightRadiusRatio: 0
	borderWidth: 1
	borderColor: colors.tempTileBorderUp
	borderStyle: Qt.SolidLine

	mouseIsActiveInDimState: true

	shadowColor: "#00000000"

	Text {
		id: subLabel
		anchors.verticalCenter: parent.verticalCenter
		anchors.horizontalCenter: parent.horizontalCenter
		font.family: qfont.semiBold.name
		font.pixelSize: qfont.tileTitle
	}

	state: "up"

	states: [
		State {
			name: "up"
			PropertyChanges { target: root;     color: colors.tempTileBackgroundUp }
			PropertyChanges { target: root;     shadowPixelSize: 1 }
			PropertyChanges { target: root;     borderColor: colors.tempTileBorderUp }
			PropertyChanges { target: subLabel; color: colors.tempTileTextUp }
		},
		State {
			name: "down"
			PropertyChanges { target: root;     color: colors.tempTileBackgroundDown }
			PropertyChanges { target: root;     shadowPixelSize: 0 }
			PropertyChanges { target: root;     borderColor: colors.tempTileBorderDown }
			PropertyChanges { target: subLabel; color: colors.tempTileTextDown }
		},
		State {
			name: "unselected"
			PropertyChanges { target: root;     color: colors.tempTileBackgroundUp }
			PropertyChanges { target: root;     shadowPixelSize: 1 }
			PropertyChanges { target: root;     borderColor: colors.tempTileBorderUp }
			PropertyChanges { target: subLabel; color: colors.tempTileTextUp }
		},
		State {
			name: "selected"
			PropertyChanges { target: root;     color: colors.tempTileBackgroundDown }
			PropertyChanges { target: root;     shadowPixelSize: 0 }
			PropertyChanges { target: root;     borderColor: colors.tempTileBorderDown }
			PropertyChanges { target: subLabel; color: colors.tempTileTextDown }
		},
		State {
			name: "disabled"
			PropertyChanges { target: root;     color: colors.tempTileBackgroundDown }
			PropertyChanges { target: root;     shadowPixelSize: 0 }
			PropertyChanges { target: root;     borderColor: colors.tempTileBorderDown }
			PropertyChanges { target: subLabel; color: "lightgray" }
		}
	]

	onSelectedChanged: {
		if (selected) {
			root.state = "selected"
		} else {
			root.state = "unselected"
		}
	}

	onEnabledChanged: {
		if (!enabled) {
			root.state = "disabled"
		}
		else {
			if (selected) {
				root.state = "selected"
			} else {
				root.state = "unselected"
			}
		}
	}
}
