import QtQuick 1.1

import qb.base 1.0
import qb.components 1.0
import BasicUIControls 1.0;

SidePanelButton {
	id: thermostatSidePanelButton

	property string kpiPostfix: "thermostatSidePanelButton"

	property ThermostatApp app
	panelUrl: app.sidePanelUrl

	QtObject {
		id: p

		property string flameIconState: "off"

		//for unit tests
		property alias flameIconSource : flameIcon.source

		function onThermostatDatasetsChanged() {
			var flameState = "off";

			var modulationLevel = app.thermInfo['currentModulationLevel'];
			// burnerInfo 0=off, 1=heat, 2=water, 3=preheat, 4=error
			var burnerInfo = app.thermInfo['burnerInfo'];
			//When burner is set to on (heating or preheating) use the modulation level as a guide
			switch (burnerInfo) {
			case 1:
			case 3:
				//Show flames based on modulation level
				if (modulationLevel > 70) {
					flameState = "3";
				} else if (modulationLevel > 35) {
					flameState = "2";
				} else {
					flameState = "1";
				}
				break;
			case 2:
				//Heating water now handled by DHW panel
				flameState = "hw";
				break;
			case 4:
				//Error
				flameState = "error";
				break;
			}

			if (app.thermInfo.boilerModuleConnected == 0) {
				flameState = "error";
			} else if (app.thermInfo.haveOTBoiler == "1" && app.thermInfo.otCommError == "1") {
				flameState = "error";
			} else if (app.thermInfo.haveOTBoiler == "1" && app.thermInfo.errorFound != 255 && !app.boilerManagementFeature) {
				flameState = "error";
			} else if (app.boilerManagementFeature && app.boilerErrorInfo.errorActive) {
				flameState = "error";
			}

			if (!app.isUndef(app.heatRecoveryInfo)) {
				if (app.heatRecoveryInfo["BlockingState"] || app.heatRecoveryInfo["CurrentFaultcode"] || !app.heatRecoveryInfo["IsConnected"]) {
					flameState = "heatwinner-error";
				} else if (app.heatRecoveryInfo["CurrentState"]) {
					if (flameState === "off") {
						flameState = "heatwinner";
					} else if (flameState === "hw") {
						flameState = "heatwinner-hw";
					} else if (flameState !== "error") {
						flameState = "heatwinner-boiler";
					}
				}
			}

			p.flameIconState = flameState;
		}
	}

	function init() {
		app.thermInfoChanged.connect(p.onThermostatDatasetsChanged);
		app.boilerErrorInfoChanged.connect(p.onThermostatDatasetsChanged);
		app.boilerManagementFeatureChanged.connect(p.onThermostatDatasetsChanged);
		app.heatRecoveryInfoChanged.connect(p.onThermostatDatasetsChanged);
	}

	Image {
		id: flameIcon
		source: "drawables/ts-" + (canvas.dimState ? "dim" : thermostatSidePanelButton.state) + "-" + p.flameIconState + ".svg"

		anchors {
			horizontalCenter: parent.horizontalCenter
			verticalCenter:   parent.verticalCenter
			verticalCenterOffset: canvas.dimState ? 60 : 0
		}
	}

	Connections {
		target: canvas
		onDimStateChanged: {
			if (canvas.dimState) {
				sendShowPanel();
				flameIcon.anchors.horizontalCenter = undefined;
				flameIcon.anchors.left = thermostatSidePanelButton.left
			} else {
				flameIcon.anchors.left = undefined;
				flameIcon.anchors.horizontalCenter = thermostatSidePanelButton.horizontalCenter;
			}
		}
	}

}
