
import QtQuick 1.1
import qb.components 1.0
import "../../themes"

/// Overview screen for vacation.

Screen {
	id: vacationOverviewScreen
	screenTitleIconUrl: "./drawables/vacation.svg"
	screenTitle: qsTr("Vacation")

	anchors.verticalCenter: parent.verticalCenter
	anchors.horizontalCenter: parent.horizontalCenter

	QtObject {
		id: p

		function onVacationDataChanged() {
			txtTempValue.text = i18n.number(app.vacationData['temperature'], 1) + "°";
			txtFromValue.text = app.formatDateTime(app.vacationData['startTime'], true);
			txtToValue.text = app.formatDateTime(app.vacationData['endTime'], false);
			if (!app.hasVacation)
				vacationDataSavedText.visible = false;
		}
	}

	function vacationDataSet() {
		vacationDataSavedText.visible = true;
	}

	onShown: {
		app.vacationDataChanged.connect(p.onVacationDataChanged);
		app.vacationSet.connect(vacationDataSet);
		p.onVacationDataChanged();
	}

	onHidden: {
		vacationDataSavedText.visible = false;
		app.vacationDataChanged.disconnect(p.onVacationDataChanged);
	}

	Image {
		id: vacationImg
		source: "drawables/vacation.svg"
		height: 86
		sourceSize.height: height
		anchors {
			horizontalCenter: parent.horizontalCenter
			top: parent.top
			topMargin: Math.round((app.hasVacation ? 70 : 99) )
		}
	}

	Item {
		id: itemNoVacation
		visible: !app.hasVacation
		anchors {
			top: vacationImg.bottom
			horizontalCenter: parent.horizontalCenter
		}

		Text {
			id: txtNoVacation
			text: qsTr("No vacation scheduled yet")
			color: "#565656"
			anchors {
				horizontalCenter: parent.horizontalCenter
				baseline: parent.top
				baselineOffset: 36
			}
			font {
				family: qfont.semiBold.name
				pixelSize: qfont.navigationTitle
			}
		}

		StandardButton {
			id: butSetVacation
			text: qsTr("Set")
			anchors {
				top: txtNoVacation.baseline
				topMargin: 22
				horizontalCenter: parent.horizontalCenter
			}
			onClicked: {
				stage.openFullscreen(app.vacationSetScreenUrl, {newVacation: true});
			}
		}
	}

	Item {
		id: vacationSet
		visible: app.hasVacation
		width: childrenRect.width
		height: childrenRect.height
		anchors {
			top: vacationImg.bottom
			horizontalCenter: parent.horizontalCenter
		}

		Text {
			id: txtTemperature
			text: qsTr("Set to")
			width: 169
			font {
				family: qfont.semiBold.name
				pixelSize: qfont.navigationTitle
			}
			anchors {
				left: parent.left
				baseline: parent.top
				baselineOffset: 43
			}
			color: "#565656"
		}
		Text {
			id: txtFrom
			text: qsTr("From")
			width: 169
			font {
				family: qfont.semiBold.name
				pixelSize: qfont.navigationTitle
			}
			anchors {
				left: txtTemperature.right
				leftMargin: 26
				top: txtTemperature.top
			}
			color: "#565656"
		}
		Text {
			id: txtTo
			text: qsTr("To")
			width: 169
			font {
				family: qfont.semiBold.name
				pixelSize: qfont.navigationTitle
			}
			anchors {
				left: txtFrom.right
				leftMargin: 26
				top: txtTemperature.top
			}
			color: "#565656"
		}

		Rectangle {
			id: recTemperature
			width: 169
			height: 36

			color: "#ffffff"
			radius: 2
			anchors {
				left: txtTemperature.left
				top: txtTemperature.baseline
				topMargin: 10
			}
			Text {
				id: txtTempValue
				objectName: "vacationOverviewScreen_temperatureValue"
				text: i18n.number(app.vacationData.temperature, 1) + "°"
				anchors {
					fill: parent
					leftMargin: 8
				}
				font {
					family: qfont.regular.name
					pixelSize: qfont.titleText
				}
				color: "#565656"
				horizontalAlignment: Text.AlignLeft
				verticalAlignment: Text.AlignVCenter
			}
		}

		Rectangle {
			id: recFrom
			width: 169
			height: 36
			color: "#ffffff"
			radius: 2
			anchors {
				left: txtFrom.left
				top: recTemperature.top
			}
			Text {
				id: txtFromValue
				objectName: "vacationOverviewScreen_fromValue"
				text: app.formatDateTime(app.vacationData.startTime, true)
				height: parent.height; width: parent.width
				anchors {
					left: parent.left
					right: parent.right
					leftMargin: 8
					rightMargin: 2
				}
				font {
					family: qfont.regular.name
					pixelSize: qfont.titleText
				}
				color: "#565656"
				horizontalAlignment: Text.AlignLeft
				verticalAlignment: Text.AlignVCenter
			}
		}

		Rectangle {
			id: recTo
			width: 169
			height: 36
			color: "#ffffff"
			radius: 2
			anchors {
				left: txtTo.left
				top: recFrom.top
			}
			Text {
				id: txtToValue
				objectName: "vacationOverviewScreen_untilValue"
				text: app.formatDateTime(app.vacationData.endTime, false)
				height: parent.height
				anchors {
					left: parent.left
					right: parent.right
					leftMargin: 8
					rightMargin: 2
				}
				font {
					family: qfont.regular.name
					pixelSize: qfont.titleText
				}
				color: "#565656"
				horizontalAlignment: Text.AlignLeft
				verticalAlignment: Text.AlignVCenter
			}
		}

		IconButton {
			id: butEdit
			objectName: "vacationOverviewScreen_butEdit"
			iconSource: '../../images/edit.svg'
			anchors {
				left: recTo.right
				leftMargin: 6
				top: recTo.top
			}
			onClicked: {
				stage.openFullscreen(app.vacationSetScreenUrl, {editVacation: true});
			}
		}

		IconButton {
			id: butDelete
			iconSource: '../../images/delete.svg'
			anchors {
				left: butEdit.right
				leftMargin: 6
				top: butEdit.top
			}
			onClicked: app.abortVacation()
		}
	}

	WarningBox {
		id: vacationDataSavedText
		width: 455
		height: 45
		warningText: qsTr("Your vacation is set and saved.")
		warningIcon: './drawables/vacation.svg'
		anchors {
			horizontalCenter: parent.horizontalCenter
			top: vacationSet.bottom
			topMargin: 72
		}
		visible: false
	}
}
